<?php
namespace Controller;

use Util\Utilities;
use Util\Messages;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;

class CalendarDetail extends \Generated\Controller\CalendarDetail
{

    /**
     * @var \Util\Utilities
     */
    private $environment;
    /**
     * @var \Service\CalendarDetail
     */
    private $calendarDetailService;

    public function __construct(Manager $dataSourceManager, \Util\Utilities $environment,
                                \Service\CalendarDetail $calendarDetailService)
    {
        parent::__construct($dataSourceManager);
        $this->environment = $environment;
        $this->calendarDetailService = $calendarDetailService;
    }

    public function find(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $calendar = Utilities::getValueInFilterDataSourceByName($filters,'CALENDAR', true);
        $id = Utilities::getValueInFilterDataSourceByName($filters,'ID', true);
        $filterString = Utilities::createQueryFiltered($filters);
        $dataSet = $this->calendarDetailService->findCalendarException($calendar,$id, $filterString);
        $response->addDataset(new DataSet('CalendarDetail', $dataSet));
    }
    
    public function findByStructure(Request\Filter $request, Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $structure = Utilities::getValueInFilterDataSourceByName($filters,'STRUCTURE');
        $id = Utilities::getValueInFilterDataSourceByName($filters,'ID');
        $dataSet = array();
        if($structure){
            $dataSet = $this->calendarDetailService->findCalendarExceptionByStructure($structure,$id);
        }
        $response->addDataset(new DataSet(explode('/',$request->getRoutePath())[1], $dataSet));
    }

    public function save(Request $request, Response $response,$showMessage = true)
    {
        $rows = $request->getDataSet()->getRows();
        
        foreach ($rows as &$row) {
            list($recurrenceType, $calendar, $id, $type, $status, $name, $beginDate, $occurrences,
                $exceptionQuantitative, $exceptionDataEnd, $endDate, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat,
                $yearDayException, $monthlyDay, $monthly, $weekDayException, $orderDay, $weekDay, $yearDay, $workTime,
                $isApplyChildrenProject, $projectId)
                = self::builderExceptionRow($row);
            $calendarDetail = $this->calendarDetailService->beforeSaveException($id,$recurrenceType,$name,$calendar,$type,$status,$beginDate,$occurrences,
                $exceptionQuantitative,$exceptionDataEnd,$endDate,$every,$sun,$mon,$tue,$wed,$thu,$fri,$sat,$yearDayException,$monthlyDay,
                $monthly,$weekDayException,$orderDay,$weekDay,$yearDay,$workTime, $isApplyChildrenProject, $projectId);

            $dataSet = array("ID" => $calendarDetail->getId());
            $response->addDataset(new DataSet('calendarDetail', $dataSet));
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
        }
    }

    public function delete(Request $request, Response $response,$showMessage = true)
    {
        $dataSet = $request->getDataSet()->getRows();
        foreach ($dataSet as $calendarDetail ){
            $this->calendarDetailService->removeCalendarDetail($calendarDetail['ID']);
        }
        
        $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
    }

    public function copyCalendarDetail(Request\DataSet $request, Response $response){
        set_time_limit(180);
        $row = $request->getDataSet()->getRows()[0];
        $calendarReceiverIds = $row["calendarReceiverId"];
        $selectedHolidays = $row["selectedHolidays"];

        foreach ($calendarReceiverIds as $calendarReceiverId) {
            foreach ($selectedHolidays as $selectedHoliday) {
                $rowToBuild = $this->calendarDetailService->findCalendarException($selectedHoliday['CALENDAR'],$selectedHoliday['ID'])[0];
                $rowToBuild = $this->prepareRow($rowToBuild);
                list($recurrenceType, $calendar, $id, $type, $status, $name, $beginDate, $occurrences,
                    $exceptionQuantitative, $exceptionDataEnd, $endDate, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat,
                    $yearDayException, $monthlyDay, $monthly, $weekDayException, $orderDay, $weekDay, $yearDay, $workTime)
                    = self::builderExceptionRow($rowToBuild);
                $id=null;
                $this->calendarDetailService->saveException($id,$recurrenceType,$name,$calendarReceiverId,$type,$status,$beginDate,$occurrences,
                    $exceptionQuantitative,$exceptionDataEnd,$endDate,$every,$sun,$mon,$tue,$wed,$thu,$fri,$sat,$yearDayException,$monthlyDay,
                    $monthly,$weekDayException,$orderDay,$weekDay,$yearDay,$workTime);
            }
        }
    }

    public function prepareRow($rowToBuild){
        if(empty($rowToBuild['EXCEPTION_QUANTITATIVE'])){
            $rowToBuild['EXCEPTION_QUANTITATIVE']="N";
        }
        if(empty($rowToBuild['EXCEPTION_DATA_END'])){
            $rowToBuild['EXCEPTION_DATA_END']="N";
        }
        if(empty($rowToBuild['SUN'])){
            $rowToBuild['SUN']="N";
        }
        if(empty($rowToBuild['MON'])){
            $rowToBuild['MON']="N";
        }
        if(empty($rowToBuild['TUE'])){
            $rowToBuild['TUE']="N";
        }
        if(empty($rowToBuild['WED'])){
            $rowToBuild['WED']="N";
        }
        if(empty($rowToBuild['THU'])){
            $rowToBuild['THU']="N";
        }
        if(empty($rowToBuild['FRI'])){
            $rowToBuild['FRI']="N";
        }
        if(empty($rowToBuild['SAT'])){
            $rowToBuild['SAT']="N";
        }
        if(empty($rowToBuild['MONTHLY_DAY'])){
            $rowToBuild['MONTHLY_DAY']=1;
        }
        if(empty($rowToBuild['MONTHLY'])){
            $rowToBuild['MONTHLY']=1;
        }
        if(empty($rowToBuild['YEAR_DAY'])){
            $rowToBuild['YEAR_DAY']=null;
        }
        if(empty($rowToBuild['WORKTIME'])){
            $rowToBuild['WORKTIME']="8H";
        }
        if(empty($rowToBuild['YEAR_DAY_EXCEPTION_YEAR'])){
            $rowToBuild['YEAR_DAY_EXCEPTION_YEAR']=null;
        }
        if(empty($rowToBuild['WEEK_DAY_EXCEPTION_YEAR'])){
            $rowToBuild['WEEK_DAY_EXCEPTION_YEAR']=null;
        }
        if(empty($rowToBuild['WEEK_DAY_YEAR'])){
            $rowToBuild['WEEK_DAY_YEAR']=null;
        }
        if(empty($rowToBuild['WEEK_DAY_MONTH'])){
            $rowToBuild['WEEK_DAY_MONTH']=null;
        }
        if(empty($rowToBuild['EVERY_MONTH_WEEK'])){
            $rowToBuild['EVERY_MONTH_WEEK']=null;
        }
        if(empty($rowToBuild['ORDER_DAY_YEAR'])){
            $rowToBuild['ORDER_DAY_YEAR']=1;
        }
        if(empty($rowToBuild['ORDER_DAY_MONTH'])){
            $rowToBuild['ORDER_DAY_MONTH']=1;
        }
        if(empty($rowToBuild['YEAR_DAY_EXCEPTION_MONTH'])){
            $rowToBuild['YEAR_DAY_EXCEPTION_MONTH']='N';
        }
        if(empty($rowToBuild['WEEK_DAY_EXCEPTION_MONTH'])){
            $rowToBuild['WEEK_DAY_EXCEPTION_MONTH']='N';
        }
        if(empty($rowToBuild['WEEK_DAY_MONTH'])){
            $rowToBuild['WEEK_DAY_MONTH']=1;
        }
        if(empty($rowToBuild['WEEK_DAY_YEAR'])){
            $rowToBuild['WEEK_DAY_YEAR']=1;
        }

        return $rowToBuild;
    }

    public static function builderExceptionRow($row)
    {
        $recurrenceType = $row['RECURRENCE_TYPE'];
        $calendar = $row['CALENDAR'];
        $id = $row['ID'];
        $type = $row['TYPE'];
        $status = $row['STATUS'];
        $name = $row['NAME'];
        $beginDate = $row['BEGIN_DATE'];
        $occurrences = $row['OCCURRENCES'];
        $exceptionQuantitative = $row['EXCEPTION_QUANTITATIVE'];
        $exceptionDataEnd = $row['EXCEPTION_DATA_END'];
        $endDate = $row['END_DATE'];
        $sun = $row['SUN'];
        $mon = $row['MON'];
        $tue = $row['TUE'];
        $wed = $row['WED'];
        $thu = $row['THU'];
        $fri = $row['FRI'];
        $sat = $row['SAT'];
        $monthlyDay = $row['MONTHLY_DAY'];
        $monthly = $row['MONTHLY'];
        $yearDay = $row['YEAR_DAY'];
        $workTime = $row['WORKTIME'];
        $yearDayException = null;
        $weekDayException = null;
        $orderDay = null;
        $weekDay = null;
        $every = null;
        /**
         * Tratamentos de fields que mudou no frontend
         */
        if($recurrenceType == \Service\CalendarDetail::RECURRENCE_TYPE_DAILY){
            $every = $row['EVERY_DAY'];
        }else if($recurrenceType == \Service\CalendarDetail::RECURRENCE_TYPE_WEEKLY){
            $every = $row['EVERY_WEEK'];
        }else if($recurrenceType == \Service\CalendarDetail::RECURRENCE_TYPE_MONTHLY){
            $yearDayException = $row['YEAR_DAY_EXCEPTION_MONTH'];
            $weekDayException = $row['WEEK_DAY_EXCEPTION_MONTH'];
            if($yearDayException == 'Y'){
                $monthlyDay = $row['MONTHLY_DAY'];
                $every = $row['EVERY_MONTH_YEAR'];
            }else{
                $orderDay = $row['ORDER_DAY_MONTH'];
                $weekDay = $row['WEEK_DAY_MONTH'];
                $every = $row['EVERY_MONTH_WEEK'];
            }
        }else if($recurrenceType == \Service\CalendarDetail::RECURRENCE_TYPE_YEARLY){
            $yearDayException = $row['YEAR_DAY_EXCEPTION_YEAR'];
            $weekDayException = $row['WEEK_DAY_EXCEPTION_YEAR'];
            if($yearDayException == 'Y'){
                $yearDay = $row['YEAR_DAY'];
            }else{
                $orderDay = $row['ORDER_DAY_YEAR'];
                $weekDay = $row['WEEK_DAY_YEAR'];
                $monthly = $row['MONTHLY'];
            }
        }

        $beginDate = \DateTime::createFromFormat(\Util\Utilities::FORMAT_DATE_DEFAULT,$beginDate.' 00:00:00');
        $endDate = \DateTime::createFromFormat(\Util\Utilities::FORMAT_DATE_DEFAULT,$endDate.' 00:00:00');

        $yearDay = explode(' ',$yearDay);
        if(!empty($yearDay[1])) {
            $yearDay[1] = \Util\Utilities::$ARRAY_MONTH_FOR_NUMERIC[$yearDay[1]];
        }
        
        $isApplyChildrenProject = empty($row['IS_APPLY_CHILDREN_PROJECT']) ? null : $row['IS_APPLY_CHILDREN_PROJECT'];
        $projectId              = empty($row['PROJECT_ID']) ? null : $row['PROJECT_ID'];
        
        return array($recurrenceType, $calendar, $id, $type, $status, $name, $beginDate, $occurrences,
            $exceptionQuantitative, $exceptionDataEnd, $endDate, $every, $sun, $mon, $tue, $wed, $thu, $fri, $sat,
            $yearDayException, $monthlyDay, $monthly, $weekDayException, $orderDay, $weekDay, $yearDay, $workTime,
            $isApplyChildrenProject, $projectId);
    }

}