module.exports = function(grunt) {
    var jsonPath        = 'assets/json/**/*.json';
    var utilPath        = 'assets/js/util/*.js';
    var repositoryPath  = 'assets/js/Repository/*.js';
    var servicePath     = 'assets/js/Service/*.js';
    var controllerPath  = 'assets/js/Controller/*.js';
    var specsPath       = 'specs/**/*spec*.js';
    var orderFiles      = [];
    var helperPath      = 'specs/helpers/*.js';
    var compassPath     = '';
    var libFiles = [
        'gitQueue/dist/main.min.js',
        'zhuaclfrontend/assets/dist/main.js',

        'zh-interval-component/dist/zh-interval-component.js',
        'zh-task-gantt-editor/dist/zh-task-gantt-editor.js',
        'zh-charts/dist/zh-charts.js',
        'zh-editable-treelist/dist/zh-editable-treelist.js',
        'zh-description/dist/zh-description.js',
        'zh-memory-storage/src/main.js',
        'zh-cache-bypass/src/CacheBypass.js',
        'zh-select-autocomplete/dist/zh-select-autocomplete.js',
        'zh-reports/dist/zhReports.js',
        'zh-grid-navigation/dist/GridNavigationLibrary.js',

        'zh-grid-configuration/dist/zh-grid-configuration.js',
        'zh-preferences/dist/preferences.js',
        'zh-filter-preferences/dist/filter-preferences.js',
        'zh-layout-preferences/dist/layout-preferences.js',
        'zh-view-preferences/dist/view-preferences.js',

        'zhulog/dist/main.js',
        'tek-libraries/dist/main.js'
    ];
    
    function getLibPaths() {
        var paths = [];
        libFiles.forEach(function(fileName) {
            paths.push('bower_components/' + fileName);
        });
        return paths;
    }

    grunt.initConfig({
        concat: {
            zh : {
                options: {
                    process: function(src, filepath) {
                        return '\n' + '// FILE: ' + filepath + '\n' + src;
                    }
                },
                src: ['libs/**/*.js',utilPath, repositoryPath, servicePath, controllerPath],
                dest: 'assets/dist/main.js'
            },
            buildLibs: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: getLibPaths(),
                dest: 'assets/dist/libs.js'
            },
            css: {
                options: {
                    process: function (src, filepath) {
                        return '/* FILE: ' + filepath + ' */\n' + src + '\n';
                    }
                },
                src: getCSSPaths(),
                dest: './assets/css/app.css'
            }
        },
        jshint: {
            options: {
                reporterOutput: ''
            },
            all: ['Gruntfile.js', 'lib/**/*.js', specsPath, utilPath, repositoryPath, servicePath, controllerPath]
        },
        jasmine : {
            pivotal:{
                // Your project's source files
                src : [utilPath, repositoryPath, servicePath, controllerPath],
                // Your Jasmine spec files
                options: {
                    vendor : ['libs/**/*.js', "specs/libs/*.js"],
                    specs : specsPath,
                    // Your spec helper files
                    helpers : helperPath
                }
            }
        },
        uglify: {
            options: {
                mangle: false,
                compress: false,
                report : 'min',
                // the banner is inserted at the top of the output
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    'assets/dist/main.min.js': ['libs/**/*.js', utilPath, repositoryPath, servicePath, controllerPath].concat(orderFiles),
                    'config/ZeedhiConfig.min.js': 'config/ZeedhiConfig.js',
                    'assets/js/environment.min.js': 'assets/js/environment.js',
                    'config/Config.min.js': 'config/Config.js',
                    'assets/dist/libs.min.js': 'assets/dist/libs.js',
                    'assets/js/engine.min.js': 'assets/js/engine.js',
                    'templates/diff/diff.min.js': 'templates/diff/diff.js'
                }
            }
        },
        cssmin: {
            dist: {
                files: {
                    'bower_components/zeedhi-frontend/assets/css/libs/bootstrap.min.css': 'bower_components/zeedhi-frontend/assets/css/libs/bootstrap.css',
                    'bower_components/zeedhi-frontend/assets/css/styles/styles-inflightor.min.css': 'bower_components/zeedhi-frontend/assets/css/styles/styles-inflightor.css',
                    'bower_components/zeedhi-frontend/assets/fonts/font-zeedhi/stylesheet.min.css': 'bower_components/zeedhi-frontend/assets/fonts/font-zeedhi/stylesheet.css',
                    
                    'bower_components/zh-charts/dist/zh-charts.min.css': 'bower_components/zh-charts/dist/zh-charts.css',
                    'bower_components/zh-grid-navigation/dist/style.min.css': 'bower_components/zh-grid-navigation/dist/style.css',
                    'bower_components/zh-select-autocomplete/dist/style/colors/styles-inflightor.min.css': 'bower_components/zh-select-autocomplete/dist/style/colors/styles-inflightor.css',
                    'bower_components/zh-reports/dist/style/colors/styles-inflightor.min.css': 'bower_components/zh-reports/dist/style/colors/styles-inflightor.css',
                    'bower_components/zh-preferences/dist/style/colors/styles-inflightor.min.css': 'bower_components/zh-preferences/dist/style/colors/styles-inflightor.css',
                    'bower_components/zh-grid-configuration/dist/style/colors/styles-inflightor.min.css': 'bower_components/zh-grid-configuration/dist/style/colors/styles-inflightor.css',
                    'bower_components/zh-task-gantt-editor/dist/zh-task-gantt-editor.min.css': 'bower_components/zh-task-gantt-editor/dist/zh-task-gantt-editor.css',
                    'bower_components/zh-description/src/styles/zh-description.min.css': 'bower_components/zh-description/src/styles/zh-description.css',
                    'bower_components/zh-editable-treelist/dist/zh-editable-treelist.min.css': 'bower_components/zh-editable-treelist/dist/zh-editable-treelist.css',
                    
                    'assets/css/app.min.css': 'assets/css/app.css',
                    'templates/diff/diff.min.css': 'templates/diff/diff.css'
                }
            }
        },
        watch: {
            pivotal : {
                files: [utilPath, repositoryPath, servicePath, controllerPath],
                tasks: [ 'jshint', 'uglify', 'concat']
            }
        },
        complexity: {
            generic: {
                src: [specsPath, utilPath, repositoryPath, servicePath, controllerPath, compassPath].concat(orderFiles),
                exclude: ['doNotTest.js'],
                options: {
                    breakOnErrors: true,
                    jsLintXML: 'report.xml',         // create XML JSLint-like report
                    checkstyleXML: 'checkstyle.xml', // create checkstyle report
                    errorsOnly: false,               // show only maintainability errors
                    cyclomatic: [],                  // or optionally a single value, like 3
                    halstead: [8, 13, 20],           // or optionally a single value, like 8
                    maintainability: 100,
                    hideComplexFunctions: false,     // only display maintainability
                    broadcast: false                 // broadcast data over event-bus
                }
            }
        },
        zhChanges : {
            pivotal: {}
        },

        zhIdGenerator: {
            options: {
                jsonPath: ["./assets/json"],
                menuPath: ["./assets/json/menu.json"],
                replaceIds: false
            }
        },
         // grunt-zhuacl-structure
        buildAclStruct : {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },
        jsonlint: {
                jshintStyle: {
                    src: jsonPath,
                    options: {
                    reporter: 'jshint'
                }
            }                        
        },
        buildEnvironment: {
            development: {
                configFile: "./config/template/devEnvironment.json"
            },
            production: {
                configFile: "./config/template/prodEnvironment.json"
            },
            test: {
                configFile: "./config/template/testEnvironment.json"
            },
            srcFolders: ["../backend/config", "./config", "./", "../version", "./assets/json", "./assets/js"]
        }
    });

    function getCSSPaths() {
        var paths = [];
        paths.push('./assets/css/**/*.css');
        paths.push('!./assets/css/app.css');
        paths.push('bower_components/zh-preferences/dist/style/style.css');
        return paths;
    }

    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-jasmine');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-cssmin');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-complexity');
    grunt.loadNpmTasks('grunt-zhuacl-structure');
    grunt.loadNpmTasks('grunt-zh-id-generator');
    grunt.loadNpmTasks('grunt-jsonlint');
    grunt.loadNpmTasks('grunt-zh-environment-builder');

    // Default task.
    grunt.registerTask('default', ['jshint', 'buildEnvironment', 'concat', 'uglify', 'cssmin', 'buildAclStruct']);
    grunt.registerTask('commit', ['jshint','uglify','concat'] );
    grunt.registerTask('basic', ['uglify', 'concat']);
    grunt.registerTask('buildLibs', ['concat:buildLibs']);

};